% compute lowest few hundred (thousand?) triangle Dirichlet eigenmodes
% Alex Barnett, for Peter Doyle   6/27/06, tweaked 9/18/12

% The triangle has vertices (0,0), (1,0) and (x,y) with y>0.
% The corresponding angles (in radians) are alpha, beta and gamma,
% You can specify alpha, beta. Therefore the area depends on these.
% Check that the comparison graph with Weyl's law does not jump up
% by one or more units - if it does this signifies missing mode(s).
% It takes about 1 min to run on my laptop.
clear

%alpha = pi/5; beta = pi/5;      % choose via angles
%s = cot(alpha); t = cot(beta); y = 1/(s+t); x = y*s;
x = 0.7; y = 0.8;               % ... or override via vertex location

area = y/2;
%n = 100;            % choose rough number of desired levels (will get less)
%Emax = (4*pi/area)*n; k = [4 sqrt(Emax)];  % go from bottom up

k = [100 110];

xo = (1+x)/3; yo = y/3;  % centroid to optimize the tri location
bil = sprintf('-l tri:%g:%g:%g:%g ', x, y, -xo, -yo);

dk = 0.05/area;      % change this to eg 0.02/area for more accuracy
opts.b = 30;
opts.Mo = 500;       % number of perimeter output points
opts.v = 0;

if 0   % get all eigenvalues from the bottom up to kmax
    sys = [bil '-s oyo:6:1.8:1 -u -4 1'];
[ks, prop, err, bdry, g] = gen_levels(3, 15, dk, dk, sys, opts);

opts.b = 30;
sys = [bil '-s oyo:5:2.5:1 -u -4 1'];
[ks_t, prop_t, err_t, bdry_t, g_t] = ...
gen_levels(15, 30, dk, dk, sys, opts);  
[ks, err, bdry, g] = merge_levels(ks, ks_t, err, err_t, bdry, bdry_t, g, g_t);

% We might not want to keep the 2d spatial mode info from now on...
sys = [bil '-s oyo:3:4:1 -u -4 1'];
[ks_t, prop_t, err_t, bdry_t, g_t] = gen_levels(30, kmax, dk, dk, sys, opts);  
[ks, err, bdry, g] = merge_levels(ks, ks_t, err, err_t, bdry, bdry_t, g, g_t);

else % use a k range (assumed to be above 30)
  sys = [bil '-s oyo:3:4:1 -u -4 1'];
  [ks, prop, err, bdry, g] = gen_levels(k(1), k(2), dk, dk, sys, opts);
end


ne = numel(ks);

% remove duplicates: find them first --------------------
norms = sqrt(dot(bdry.ngr,bdry.ngr,2));
diago = sum(bdry.ngr(1:end-1,:).*bdry.ngr(2:end,:), 2);
coso = diago ./ (norms(1:end-1).*norms(2:end));    % normalize to get cos angs
j_dups = find(abs(coso)>0.99);                 % adjust cutoff dep on acc
ind = ones(1,ne); ind(j_dups) = 0;
j_keep = find(ind); ne = numel(j_keep);
ks = ks(j_keep); bdry.ngr = bdry.ngr(j_keep,:); bdry.per = bdry.per(j_keep,:);
err.ten = err.ten(j_keep);err.nrm = err.nrm(j_keep);err.kos = err.kos(j_keep);
g.sta = g.sta(j_keep,:,:);
% -----------------------------------------------------------------
Es = ks.^2;

% leave off the final g if you don't want 2d mode plot information...
%save tri_pi5.mat ne ks Es err bdry prop opts sys g

save_sta(g.sta, ks, 't');  % viewer-readable (note t.* overwritten each time)

figure; semilogy(ks, err.ten, '+'); title('boundary errors'); xlabel('k');
figure; k_weyl = weyl(0, ks, prop.perim_colloc, prop.area);
Mo = opts.Mo;
figure; imagesc(1:ne, bdry.dx*(1:Mo), bdry.ngr'.^2./repmat(Es', [Mo 1]));
set(gca, 'ydir', 'normal'); xlabel('mode number j'); ylabel('bdry coord');


figure;  % show lowest eigenmodes to check.................................
for j=1:20
  subplot(4,5,j);
  imagesc(squeeze(g.sta(j,:,:) .* g.mask(1,:,:))');
  set(gca, 'ydir','normal','xtick', [],'ytick',[]);axis equal;caxis([-2 2]); 
end
%print -depsc2 tri.eps
